/**
 * Gantt configuration file
 */

import '../lib/GanttToolbar.js';
import '../lib/StatusColumn.js';
import '../lib/FinishColumn.js';
import '../lib/MyEndDate.js';
import '../lib/GroupColumn.js'

import Task from '../lib/Task.js';
import { DateHelper, Gantt, MessageDialog, SchedulerResourceModel } from '@bryntum/gantt/gantt.lite.umd.js';
import { from } from 'rxjs';
import { parseTemplate } from '@angular/compiler';

const ganttConfig = {
    minZoomLevel: 0,
    maxZoomLevel: 10,
    newTaskDefaults: (targetRecord) => {
        console.log('targetRecord', targetRecord)
        return { startDate: targetRecord.startDate, manuallyScheduled: false, duration: 1, effort: 0 }
    },
    dependencyIdField: 'wbsCode',
    listeners: {

        beforeEventDelete() {
        },
        pdfExport(data) {
            // Not OK
            console.log(data);
            // window.open(response.url)
        },
        taskMenuBeforeShow(editorContext) {



            // if (editorContext.taskRecord.planiswareTaskId) {
            //     return false
            // }
        },
        beforeTaskDropFinalize({ context }) {
            const { taskRecords } = context;
            // Don't allow dropping a task in the past
            console.log(context)
        },
        afterTaskDrop(data) {
            data.taskRecords[0]._data.schaefflerEndDate = DateHelper.format(new Date(data.context.record.endDate.getTime() - 24 * 3600000), 'MMM DD, YYYY');
            console.log(data)
        },
        taskDrop(data) {
            console.log(data)
        },
        taskDrag(data) {
            console.log(data)
        },
        beforeTaskResizeFinalize(data) {
            console.log(data)

            data.context.taskRecord._data.schaefflerEndDate = DateHelper.format(new Date(data.context.date.getTime() - 24 * 3600000), 'MMM DD, YYYY');


        },
        beforePaste(event) {
            const { referenceRecord } = event;
            if (referenceRecord.planiswareTaskId) {
                return false
            }
            // (window as any).GanttComponent.component.isPaste = true
        },
        paste(event) {
            const { records } = event;

            // records[0].set('status', '')
            console.log(records);

            (window as any).GanttComponent.component.contentSetNull(records)
                (window as any).GanttComponent.component.statusSetNull(records)
        },

        beforeCellEditStart({ editorContext }, value, ...event) {
            const { column, editor, record, cell, taskRecord } = editorContext;
            if (column.type == 'statuscolumn' && record.groupId != JSON.parse(localStorage.getItem('groupData'))[0].id && record.isMilestone) {
                return false
            }
            if (column.type == 'statuscolumn') {
                (window as any).GanttComponent.component.gantt.columns.get('status').editor.store.removeAll()

                if (!record.isMilestone) {
                    (window as any).GanttComponent.component.gantt.columns.get('status').editor.store.add([{ 'text': 'To Do', 'id': 'To Do' }, { 'text': 'Doing', 'id': 'Doing' }, { 'text': 'Cancelled', 'id': 'Cancelled' }, { 'text': 'Done', 'id': 'Done' }])
                }
                if (record.isMilestone && record.groupId == JSON.parse(localStorage.getItem('groupData'))[0].id) {
                    (window as any).GanttComponent.component.gantt.columns.get('status').editor.store.add([{ 'text': 'To Do', 'id': 'To Do' }, { 'text': 'Doing', 'id': 'Doing' }, { 'text': 'Review', 'id': 'Review' }, { 'text': 'Done', 'id': 'Done' }])

                }
            }
            if (record.dataSourceType == 1) {
                if (column.type !== 'resourceassignment') {
                    return false
                } else {
                    console.log(record.isMilestone)
                    if (record.isMilestone) {
                        return true
                    } else {
                        return false
                    }
                }
            }
            if (column.type == "finishcolumn") {
                record.schaefflerEndDate = new Date(record.endDate.getTime() - 24 * 3600000);

            }
            // if (column.type == "finishcolumn" && record.isMilestone) {
            //     return false
            // }
            if (column.text == "SAP WBS Element") {
                return false
            }
            if (column.text == "BUDGETED") {
                return false
            }
            if (column.type == "duration" && record.isMilestone) {
                return false
            }
            if (column.type === 'percentdone') {
                // prevent editing percentdone
                return false;
            }
            if (editorContext.record.isMilestone && column.type === 'effort') {
                return false;
            }
            if (column.type === 'resourceassignment' && record.isParent) {
                return false;
            }
            if (editorContext.column.constructor.type === 'effort') {
                editorContext.editor.min = '0h';
            }
            if (column.type == 'GroupColumn') {
                editor.value = ''
            }

        },

    },
    viewPreset: {
        timeResolution: {
            unit: 'day',
            increment: 1
        },
        headers: [
            {
                unit: 'week',
                renderer: (startDate, endDate) => `${DateHelper.format(startDate, 'YY') + DateHelper.format(startDate, 'WW')}`
            }
        ],
        displayDateFormat: 'YYYY-MM-DD',
    },
    readOnly: false,
    project: {
        taskStore: {
            listeners: {
                update: (event) => {
                    console.log(event.changes)
                }
            }
        },

        calendar: 'general',
        // Let the Project know we want to use our own Task model with custom fields / methods
        taskModelClass: Task,

        autoLoad: false,
        direction: 'Backward',
        // The State TrackingManager which the UndoRedo widget in the toolbar uses
        stm: {
            autoRecord: true
        },
        resetUndoRedoQueuesAfterLoad: true,
        // This config enables response validation and dumping of found errors to the browser console.
        // It's meant to be used as a development stage helper only so please set it to false for production systems.
        validateResponse: true
    },

    startDate: '2019-01-12',
    endDate: '2019-03-24',
    resourceImageFolderPath: 'https://qakimigatewaybe.schaefflercn.com/WorkImg/',
    columns: [
        { type: 'wbs' },
        {
            type: 'name', width: 250,
            htmlEncode: false,
            leafIconCls: null,
            renderer(
                { record, value, size }:
                    { record: SchedulerResourceModel & { [key: string]: any }; value: string; size: { height: number; color: string } }
            ): string {
                // Parent rows are employees
                if (record.remark) {
                    return `<span class='contentIcon'></span> ${encodeHtml(value)}`;
                } else {
                    return `<i class='b-tree-icon b-icon b-icon-tree-leaf'></i> ${encodeHtml(value)}`;
                }


            }
        },
        { type: 'startdate' },
        { type: 'enddate' },
        { type: 'effort' },
        // { type: 'finishcolumn' },
        { type: 'duration' },
        { type: 'GroupColumn' },
        {
            type: 'resourceassignment', width: 120, showAvatars: true, itemTpl: ({ resourceName }) => resourceName,

            editor: {

                picker: {
                    listeners: {
                        async selectionChange(changed) {
                            //from(iterable) —— 把它的参数转换成一个 Observable 实例。 该方法通常用于把一个数组转换成一个（发送多个值的）可观察对象。
                            const myObservable2 = from([1, 2, 3]);
                            const myObserver2 = myObservable2.subscribe(res => {
                                console.log('使用subscribe订阅的结果：', res);
                            })

                            const myObserver3 = myObservable2.toPromise().then(res => {
                                console.log('使用toPromise转化的结果：', res);
                            })

                            console.log(changed);
                            if (changed.action == "select") {
                                const result = await MessageDialog.confirm({
                                    title: 'The big question',
                                    message: 'Do you really want to select?',
                                    okButton: 'Yes',
                                    cancelButton: 'No'
                                });

                                if (result != MessageDialog.okButton) {
                                    this.deselectRow(changed.selected[0])
                                }
                            }
                            // if (changed.action == "deselect") {
                            //     const result = await MessageDialog.confirm({
                            //         title: 'The big question',
                            //         message: 'Do you really want to deselect?',
                            //         okButton: 'Yes',
                            //         cancelButton: 'No'
                            //     });

                            // if (result != MessageDialog.okButton) {
                            //     this.selectRow(changed.deselected[0])
                            // }
                            // }
                        }
                    },
                    selectionMode: {
                        rowCheckboxSelection: true,
                        multiSelect: false,
                        showCheckAll: false,

                    },

                    features: {
                        filterBar: true,
                        group: 'resource.orgCode',
                        headerMenu: false,
                        cellMenu: false,
                    },
                    // The extra columns are concatenated onto the base column set.
                    // columns: [{
                    //     text: 'Calendar',
                    //     // Read a nested property (name) from the resource calendar
                    //     field: 'resource.name',
                    //     filterable: false,
                    //     editor: false,
                    //     width: 200,
                    // }],
                },

            }
        },
        { type: 'percentdone', showCircle: true, width: 70 },
        {
            type: 'predecessor',
            width: 112
        },
        {
            type: 'successor',
            width: 112
        },
        { type: 'schedulingmodecolumn' },
        { type: 'calendar' },
        { type: 'constrainttype' },
        { type: 'constraintdate' },
        { type: 'statuscolumn' },


        {
            type: 'date',
            text: 'Deadline',
            field: 'deadline'
        },
        { type: 'addnew' }
    ],

    subGridConfigs: {
        locked: {
            flex: 3
        },
        normal: {
            flex: 4
        }
    },

    columnLines: false,

    features: {
        excelExporter: {
            // Choose the date format for date fields
            dateFormat: 'YYYY-MM-DD'
        },
        pdfExport: {
            exportServer: 'https://dev.bryntum.com:8082',
            // Development config
            // translateURLsToAbsolute : 'http://localhost:4200',
            listeners: {
                exportStep({ progress }) {
                    // OK
                    console.log(progress);
                },
                pdfExport({ response, error }) {
                    // Not OK
                    console.log(response);
                }
            },
            // For production replace with this one. See README.md for explanation
            // translateURLsToAbsolute : 'http://localhost:8080/resources/', // Trailing slash is important
            keepPathName: false
        },
        taskMenu: {
            // Extra items for all events
            items: {
                flagTask: {
                    text: 'Comments',
                    icon: 'b-fa b-fa-fw b-fa-flag',
                    onItem({ taskRecord }) {
                        console.log(taskRecord)
                        window['GanttComponent'].component.isShowCommentsOut = true
                        // taskRecord.flagged = true;
                    }
                }
            }
        },
        eventDrag: {
            // A minimal start date tooltip
            tooltipTemplate: ({ eventRecord, finishDate }) => {
                console.log(finishDate)
                // return DateHelper.format(startDate, 'HH:mm');
            }
        },
        rollups: {
            disabled: true
        },
        baselines: {
            disabled: true
        },
        progressLine: {
            disabled: true,
            statusDate: new Date(2019, 0, 25)
        },
        filter: true,
        dependencyEdit: true,
        timeRanges: {
            showCurrentTimeLine: true
        },
        labels: {
            left: {
                field: 'name',
                editor: {
                    type: 'textfield'
                },
            },


        }
    },

    tbar: {
        type: 'gantttoolbar'
    }
};

export default ganttConfig;
function encodeHtml(html: string) {
    const div = document.createElement('div');
    div.textContent = html;
    return div.innerHTML;
}
